{******************************************************************************}
{* This software is provided 'as-is', without any express or                  *}
{* implied warranty. In no event will the author be held liable               *}
{* for any damages arising from the use of this software.                     *}
{*                                                                            *}
{* Permission is granted to anyone to use this software for any               *}
{* purpose, including commercial applications, and to alter it                *}
{* and redistribute it freely, subject to the following                       *}
{* restrictions:                                                              *}
{*                                                                            *}
{*   1. The origin of this software must not be misrepresented,               *}
{*      you must not claim that you wrote the original software.              *}
{*      If you use this software in a product, an acknowledgment              *}
{*      in the product documentation would be appreciated but is              *}
{*      not required.                                                         *}
{*                                                                            *}
{*   2. Altered source versions must be plainly marked as such, and           *}
{*      must not be misrepresented as being the original software.            *}
{*                                                                            *}
{*   3. This notice may not be removed or altered from any source             *}
{*      distribution.                                                         *}
{*                                                                            *}
{*                                                                            *}
{* The initial developer is Theo Lustenberger (temp@theo.ch)                  *}
{*                                                                            *}
{******************************************************************************}

// 29.10.2004

unit uFreeTimer;

interface

uses Classes, Libc;

type

  TFreeTimer = class;

  TTimerThread = class(TThread)
  private
    FTimer: TFreeTimer;
    FInterval: Cardinal;
    FSynchronize:Boolean;
  protected
    procedure Update;
    function GetTickCount: int64;
  public
    procedure Execute; override;
    property Timer: TFreeTimer read fTimer write fTimer;
    property Interval: Cardinal read fInterval write fInterval;
    property DoSynchronize: Boolean read fSynchronize write fSynchronize;
  end;

  TFreeTimer = class(TComponent)
  private
    FTimerThread: TTimerThread;
    FInterval: Cardinal;
    FOnTimer: TNotifyEvent;
    FEnabled: Boolean;
    FSynchronize:Boolean;
    procedure SetEnabled(Value: Boolean);
    procedure SetInterval(Value: Cardinal);
    procedure SetSynchronize(Value: Boolean);    
  protected
    procedure Timer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Synchronize: Boolean read fSynchronize write SetSynchronize;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Interval: Cardinal read FInterval write SetInterval default 1000;
    property OnTimer: TNotifyEvent read FOnTimer write FOnTimer;
  end;

  procedure Register;

implementation

{ TFreeTimer }

constructor TFreeTimer.Create(AOwner: TComponent);
begin
  inherited;
  fSynchronize:=true;
  fInterval := 1000;
  fEnabled := false;
  Enabled := true;
end;


destructor TFreeTimer.Destroy;
begin
{$B-}
  if assigned(fTimerThread) and (not fTimerThread.Terminated) then
    fTimerThread.Terminate;
  inherited;
end;

procedure TFreeTimer.SetEnabled(Value: Boolean);
begin
  if Value <> fEnabled then
  begin
    fEnabled := Value;
    if not (csDesigning in ComponentState) then
      if Value then
      begin
        fTimerThread := TTimerThread.Create(true);
        fTimerThread.FreeOnTerminate := true;
        fTimerThread.Timer := Self;
        fTimerThread.Interval := Interval;
        fTimerThread.DoSynchronize:=Synchronize;
        fTimerThread.Resume;
      end else
      begin
        fTimerThread.Terminate;
      end;
  end;
end;

procedure TFreeTimer.SetInterval(Value: Cardinal);
begin
  fInterVal := Value;
  if not (csDesigning in ComponentState) then
{$B-}
    if assigned(fTimerThread) and (not fTimerThread.Terminated) then
      fTimerThread.fInterval := Value;
end;

procedure TFreeTimer.SetSynchronize(Value: Boolean);
begin
  fSynchronize := Value;
  if not (csDesigning in ComponentState) then
{$B-}
    if assigned(fTimerThread) and (not fTimerThread.Terminated) then
      fTimerThread.FSynchronize := Value;
end;

procedure TFreeTimer.Timer;
begin
  if Assigned(FOnTimer) then
  try
    FOnTimer(Self);
  except
{$B-}
    if assigned(fTimerThread) and (not fTimerThread.Terminated) then
      fTimerThread.Terminate;
  end;
end;


{ TTimerThread }

procedure TTimerThread.Execute;
var
  StartTime: int64;
begin
  inherited;
  StartTime := GetTickCount;
  repeat
    repeat
      usleep(100); // usleep is in microseconds
    until (GetTickCount - StartTime >= fInterval) or Terminated;
    if not Terminated then
    begin
      Starttime := GetTickCount;
      if fSynchronize then Synchronize(Update) else Update;
    end;
  until Terminated;
end;

function TTimerThread.GetTickCount: int64;
var tv: timeval;
begin
  gettimeofday(tv, nil);
{$RANGECHECKS OFF}
  Result := int64(tv.tv_sec) * 1000 + tv.tv_usec div 1000;
end;

procedure TTimerThread.Update;
begin
  fTimer.Timer;
end;

procedure Register;
begin
  RegisterComponents('Theos', [TFreeTimer]);
end;

end.
